/********************************************************************************************************
 * @file     AAA_24G_rf_frame.h
 *
 * @brief    for TLSR chips
 *
 * @author	 public@telink-semi.com;
 * @date     May. 12, 2018
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/
/*
 * led_rf_frame.h
 *
 *  Created on: Jan 13, 2014
 *      Author: xuzhen
 */

#ifndef _AAA_24G_RF_FRAME_H_
#define _AAA_24G_RF_FRAME_H_


#define		RF_PROTO_BYTE		0x51

enum
{
    PIPE_PARING			= 0x00,
    PIPE_MOUSE			= 0x01,
    PIPE_KEYBOARD		= 0x02,
    //PIPE_AUDIO			= 0x03,
    //PIPE_TOUCH			= 0x04,
    //PIPE_RC				= 0x05,
    PIPE_ALL   = 0X3F,
};

enum
{
    FRAME_TYPE_DEVICE		= 0x00,
    FRAME_TYPE_MOUSE		= 0x01,
    FRAME_TYPE_KEYBOARD		= 0x02,
    FRAME_TYPE_AUDIO		= 0x03,
    FRAME_TYPE_TOUCH		= 0x04,
    FRAME_TYPE_VACUUM		= 0x05,
    FRAME_TYPE_VACUUM_CONF	= 0x06,
    FRAME_TYPE_VACUUM_MANN	= 0x07,
    FRAME_TYPE_VACUUM_RESET = 0x08,
    FRAME_TYPE_PARING		= 0x10,

    FRAME_TYPE_ACK        	= 0x80,
    FRAME_TYPE_ACK_MOUSE  	= FRAME_TYPE_ACK | FRAME_TYPE_MOUSE,
    FRAME_TYPE_ACK_KEYBOARD	= FRAME_TYPE_ACK | FRAME_TYPE_KEYBOARD,
    FRAME_TYPE_ACK_AUDIO	= FRAME_TYPE_ACK | FRAME_TYPE_AUDIO,
    FRAME_TYPE_ACK_TOUCH	= FRAME_TYPE_ACK | FRAME_TYPE_TOUCH,
    FRAME_TYPE_ACK_VACUUM	= FRAME_TYPE_ACK | FRAME_TYPE_VACUUM,	//0x85
    FRAME_TYPE_ACK_EMPTY	= FRAME_TYPE_ACK | BIT(6),

    FRAME_TYPE_DEBUG		= 0x40,
    FRAME_TYPE_MAX,			//0x41

    FRAME_PAIR_FAIL			= 0x20,
    FRAME_NO_PAIR_FLOW		= 0x30,

    FRAME_TYPE_REMOTE_KEYBOARD = 	BIT(5),
    FRAME_TYPE_REMOTE_MIC = 		BIT(6),

    FRAME_TYPE_MOUSE_SEND_ID	 = 0x09,  //  FRAME_TYPE_MOUSE | 0x08
    FRAME_AUTO_ACK_MOUSE_ASK_ID  = 0x89,  //  FRAME_TYPE_ACK_MOUSE | 0x08

    FRAME_TYPE_KB_SEND_ID	 = 0x0a,  // FRAME_TYPE_KEYBOARD | 0x08
    FRAME_AUTO_ACK_KB_ASK_ID  = 0x8a, //FRAME_TYPE_ACK_KEYBOARD | 0x08
};

enum
{
    PKT_FLOW_CAL 	 = BIT(0),
    PKT_FLOW_SEARCH	 = BIT(2),
    PKT_FLOW_ACK_REQ = BIT(3),
    PKT_FLOW_PARING	 = BIT(4),
    PKT_FLOW_TOKEN	 = BIT(6),
    PKT_FLOW_DIR	 = BIT(7),
};

typedef struct
{
    u8 btn;
    s16 x;
    s16 y;
    s8 wheel;
} mouse_data_t;

typedef struct{
	u8 cnt;
	u8 ctrl_key;
	u8 keycode[6]; //6
	//u8 padding[2];	//  for  32 bit padding,  if KB_RETURN_KEY_MAX change,  this should be changed
}kb_data_t;

typedef struct
{
	u32 did;

	u8 key[12];
}AES_KEY;

typedef struct
{
    u32 dma_len;//won't be a fixed number as previous, should adjust with the mouse package number

    u8  rf_len;
    u8	proto;
    u8	flow;
    u8	type;

    u8	rssi;
    u8	per;
    u8	seq_no;
    u8	rsvd;

    u32 did;      //key byte0 ~byte 3

	u8  enc_key[12];  //key byte4~byte 15

} rf_packet_pairing_t;

typedef struct{
	u32 dma_len;            //won't be a fixed number as previous, should adjust with the mouse package number
	u8  rf_len;
	u8	proto;
	u8	flow;
	u8	type;
	
	u8	rssi;
	u8	per;
	u16	tick;
	u8	chn;
	u8	info0;
	u8	info1;
	u8	info2;
	u32 gid1;		//pipe1 code,	used as sync code for data pipe in hamster
	u32 did;
	u8  enc_key[12];  //key byte4~byte 15
}rf_packet_ack_pairing_t;
typedef struct{
    u32 dma_len;          

    u8  rf_len;
    u8	proto;
    u8	flow;
    u8	type;

    u8	rssi;
    u8	per;
    u16	tick;
    u8	chn;
    u8	info0;
    u8	info1;
    u8	info2;

    u32 gid1;		//dongle id
    u32 did;
	u8  enc_key[12];  //key byte4~byte 15

}rf_packet_debug_t;

typedef struct{
	u32 dma_len;            //won't be a fixed number as previous, should adjust with the mouse package number
	u8  rf_len;
	u8	proto;
	u8	flow;
	u8	type;
	u8	rssi;
	u8	per;
	u16	tick;
	u8	chn;
}rf_ack_empty_t;
typedef struct
{
   u32 device_id;
   u32 seq_no;
   u8 buf[20];
   u8 rand_dat[4];
   
}ENC_RF_DATA;

typedef struct
{
    u32 dma_len; 

    u8  rf_len;
    u8	proto;
    u8	flow;
    u8	type;

    u8	rssi;
    u8	per;
    u8	seq_no;
    u8	pno;

    u32 did;

    ENC_RF_DATA dat; 

} rf_packet_mouse_t;

typedef struct{
	u32 dma_len;            //won't be a fixed number as previous, should adjust with the mouse package number

	u8  rf_len;
	u8	proto;
	u8	flow;
	u8	type;

	u8	rssi;
	u8	per;
	u16	tick;
	u8	chn;
    u8  info;
}rf_packet_ack_mouse_t;
typedef struct {
    u32 dma_len;            

    u8  rf_len;
    u8	proto;
    u8	flow;
    u8	type;

    u8	rssi;
    u8	per;
    u8	seq_no;
    u8	pno;

    u32 did;

    ENC_RF_DATA dat; 

} rf_packet_keyboard_t;

typedef struct{
    u32 dma_len;

    u8  rf_len;
    u8	proto;
    u8	flow;
    u8	type;



	u8	rssi;
	u8	per;
	u16	tick;
    
	u8	chn;
    u8	status;
} rf_packet_ack_keyboard_t;

#endif /* LED_RF_FRAME_H_ */

