/********************************************************************************************************
 * @file     main.c 
 *
 * @brief    This is the source file for TLSR8278
 *
 * @author	 Driver Group
 * @date     May 8, 2018
 *
 * @par      Copyright (c) 2018, Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *           The information contained herein is confidential property of Telink
 *           Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *           of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *           Co., Ltd. and the licensee or the terms described here-in. This heading
 *           MUST NOT be removed from this file.
 *
 *           Licensees are granted free, non-transferable use of the information in this
 *           file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 * @par      History:
 * 			 1.initial release(DEC. 26 2018)
 *
 * @version  A001
 *
 *******************************************************************************************************/

#include "AAA_project_config.h"

extern void user_init();
extern void main_loop (void);

void generateRandomNum(int len, unsigned char *data)
{
	int i;
	unsigned int randNums = 0;
    /* if len is odd */
	for (i=0; i<len; i++ ) {
		if( (i & 3) == 0 ){
			randNums = rand();
		}

		data[i] = randNums & 0xff;
		randNums >>=8;
	}
}

/**
 * @brief		This function serves to handle the interrupt of MCU
 * @param[in] 	none
 * @return 		none
 */
_attribute_ram_code_ void irq_handler(void)
{
	u32 src = reg_irq_src;

	if(src & FLD_IRQ_TMR1_EN){
		irq_host_timer1();
		reg_tmr_sta = FLD_TMR_STA_TMR1;//write 1 to clear
	}

	unsigned short rf_irq_src = rf_irq_src_get();
    if (rf_irq_src) {
        if (rf_irq_src & FLD_RF_IRQ_TX) {
        	
            rf_tx_irq_handle();
            rf_irq_clr_src(FLD_RF_IRQ_TX);
        }

        if (rf_irq_src & FLD_RF_IRQ_RX) {
        	rf_rx_irq_handle();
            rf_irq_clr_src(FLD_RF_IRQ_RX);
        }
        
      
    }
}
void dongle_led_btn_init()
{
#if USER_DEBUG
	gpio_set_output_en(D1_LED_PIN,1);
	gpio_set_output_en(D2_LED_PIN,1);
	gpio_set_output_en(D3_LED_PIN,1);
	gpio_set_output_en(D4_LED_PIN,1);
	gpio_set_output_en(D5_LED_PIN,1);
#endif
}
/**
 * @brief		This is main function
 * @param[in]	none
 * @return      none
 */
int main (void)   //must on ramcode
{
	blc_pm_select_internal_32k_crystal();
	cpu_wakeup_init(LDO_MODE, EXTERNAL_XTAL_24M);

	int deepRetWakeUp = pm_is_MCU_deepRetentionWakeup();  //MCU deep retention wakeUp


	gpio_init(!deepRetWakeUp);
	dongle_led_btn_init();

	clock_init(SYS_CLK);

	random_generator_init();
	
   custom_init();

	user_init();

	while (1) {
		main_loop ();
	}
	return 0;
}
