/********************************************************************************************************
 * @file     app_config.h 
 *
 * @brief    This is the header file for TLSR8278
 *
 * @author	 Driver Group
 * @date     May 8, 2018
 *
 * @par      Copyright (c) 2018, Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *           The information contained herein is confidential property of Telink
 *           Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *           of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *           Co., Ltd. and the licensee or the terms described here-in. This heading
 *           MUST NOT be removed from this file.
 *
 *           Licensees are granted free, non-transferable use of the information in this
 *           file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 * @par      History:
 * 			 1.initial release(DEC. 26 2018)
 *
 * @version  A001
 *         
 *******************************************************************************************************/
#pragma once
/* Enable C linkage for C++ Compilers: */
#if defined(__cplusplus)
extern "C" {
#endif

#include "AAA_app_config.h"
#include "application/usbstd/usb.h"
#include "AAA_24G_rf_frame.h"




#define U32_MAX ((u32)0xffffffff)
#define U16_MAX ((u16)0xffff)
#define U8_MAX ((u8)0xff)
#define U31_MAX ((u32)0x7fffffff)
#define U15_MAX ((u16)0x7fff)
#define U7_MAX ((u8)0x7f)
#define abs_aaa(a)   (((a)>0)?((a)):(-(a)))

#define   CFG_ADR_MAC                   0x1F000
#define   MOUSE_PARING_INFO_ADDR        0x0A000
#define   KB_PARING_INFO_ADDR           0x0B000

//#define   SAVE_MULIT_MOUSE_KB_ID_COUNT   1  //max 8

#define MOUSE_PIPE1_DATA_WITH_DID			1
#define KEYBOARD_PIPE1_DATA_WITH_DID		1

#define  PAIR_CHECK_RSSI_ENABLE_AAA     0
#define  GLOD_DONGLE_RSSI_ENABLE_AAA     1
#define  GET_RX_RSSI(P)  (P[P[0]+2])
/***************rf ll************/
#define	CHANNEL_HOPPING_ALWAYS				1
#define	LL_HOST_RX_MULTI_RECEIVING_EN		0

#define		PKT_BUFF_SIZE		80

#define		FH_CHANNEL_PKT_RCVD_MAX			8
#define		LL_CHANNEL_SYNC_TH				2
#define		LL_CHANNEL_SEARCH_TH			60
#define		LL_CHANNEL_SEARCH_FLAG			BIT(16)

#define		LL_NEXT_CHANNEL(c)				((c + 6) & 14)


#define  FRQ_HOPPING_MODE_SWITCH_ENABLE    	1

#define  FRQ_HOPPING_MODE_NORMAL   		0  //normal hopping
#define  FRQ_HOPPING_MODE_SLOW     		1  //slow hopping for kb only

#define  PKT_RCVD_FLG_MOUSE_LOCAL     	0x01
#define  PKT_RCVD_FLG_MOUSE_GLOBAL    	0x10
#define  PKT_RCVD_FLG_MOUSE				(PKT_RCVD_FLG_MOUSE_LOCAL | PKT_RCVD_FLG_MOUSE_GLOBAL)
#define  PKT_RCVD_FLG_KB_LOCAL       	0x02
#define  PKT_RCVD_FLG_KB_GLOBAL      	0x20
#define  PKT_RCVD_FLG_KB                (PKT_RCVD_FLG_KB_LOCAL | PKT_RCVD_FLG_KB_GLOBAL)

#define  PKT_RCVD_FLG_MOUSE_KB_LOCAL	(PKT_RCVD_FLG_MOUSE_LOCAL | PKT_RCVD_FLG_KB_LOCAL)

#define  MOUSE_PKT_NEVER_COME   (!(frq_hopping_data.device_pktRcv_flg & PKT_RCVD_FLG_MOUSE_GLOBAL))
#define  MOUSE_PKT_COME           (frq_hopping_data.device_pktRcv_flg & PKT_RCVD_FLG_MOUSE_LOCAL)
#define  KB_PKT_COME           	  (frq_hopping_data.device_pktRcv_flg & PKT_RCVD_FLG_KB_LOCAL)
#define  MOUSE_PKT_5S_NO_COME	  (clock_time_exceed(frq_hopping_data.mouse_pktRcv_sysTick,5000000))

typedef struct{
	u8 frq_hp_mode;
	u8 frq_hp_chn_pkt_rcvd_max;
	u8 frq_hp_hit_diff_num;
	u8 device_pktRcv_flg;

	u32 fre_hp_always_time_us;
	u32 mouse_pktRcv_sysTick;
}frqHopping_data_t;
extern frqHopping_data_t  frq_hopping_data;
/***********************************/
typedef enum
{
   MANUAL_PARING_TYPE=0Xfb,
   AUTO_LIMIT_TIME_PARING_TYPE=0xa7,
   GOLDEN_DONGLE_PARING_TYPE=0x6f,
	
}PAIRING_TYPE_AAA;
typedef struct{
	u32  mac_addr;		  		  // 00~0x03 device id
	
	u16  manufacturer_addr;		  // 04~0x05 device id
	u16  rf_id;		  		  // 06~0x07 vendor id
	
	u8	 paring_type;	   // 08 paring_type
	u8   paring_limit_t;   // 09 manual_paring_time_set 	
	u8	 support_type;     // 0a	 
	u8	 rssi_threshold;   // 0b
	
	u8   channal_msk;      // 0c
	u8 	 report_type;      // 0d  default for mouse report rate 125,other for 250
	u16  vendor_id;        // 0e-0f
	
	u16  prodct_id;        // 10-11
	u8   tx_power_emi;     // 12
	u8   ms_kb_mcu_type;   // 13 
	
	u8   bond_count;    // 14 
    u8   rsvd_1;       //15  //for 4 byte  alignment
	u8   rsvd_2;       //16
	u8   rsvd_3;       //17
	
	u8  encKey[16];     //18~27 aes128 key 16bytes

} custom_cfg_t;
extern custom_cfg_t user_config;

/***********************************/
typedef enum
{
   NORMAL_KB_DATA_TYPE=1,
   MOUSE_DATA_TYPE=2,
   CONSUME_DATA_TYPE=3,
   SYSTEM_DATA_TYPE=4,  
}USB_DATA_TYPE_ENUM;
typedef enum
{
	SUPPORT_KEYBOARD_DEVICE=0,
}SUPPORT_DEVIE_TYPE;

#define USB_FIFO_NUM   8 
#define USB_FIFO_MAX_LEN  9
typedef struct
{
	u8 fifo[USB_FIFO_NUM][USB_FIFO_MAX_LEN];
	u8 wptr;
	u8 rptr;
}USB_FIFO_DATA_S;
extern USB_FIFO_DATA_S  usb_fifo_aaa;
typedef struct
{
	u8 type;
	u8 buf[USB_FIFO_MAX_LEN-1];
	
}USB_DATA_S;

#define RF_RX_FIFO_NUM   8
#define RF_RX_FIFO_MAX_LEN  64
typedef struct
{
	u8 fifo[RF_RX_FIFO_NUM][RF_RX_FIFO_MAX_LEN];
	u8 wptr;
	u8 rptr;
}RF_RX_FIFO_DATA_S;

#define MAX_STRING_LEN  22

typedef struct
{
	u8 Size;
	u8 Type;
	wchar_t str[MAX_STRING_LEN];
}KM_USB_Descriptor_String_t;

extern u8 usb_mouse_report_proto;

extern  u8	host_keyboard_status;
extern  u32 resume_host_tick;
extern  u8 allow_send_to_usb_flag;
extern  u8 need_wake_up_host_flag;
extern  u32 tick_resume_host;

extern u8 emi_flg;


extern  void irq_host_timer1();
extern  void rf_rx_irq_handle();
extern  void rf_tx_irq_handle();
extern void custom_init();
int usb_keyboard_hid_report_aaa(unsigned char *data);
int usb_mouse_hid_report_aaa(u8 report_id,unsigned char * p,u8 len);
void usb_power_down_handle();
void usb_check_allow_send_to_usb();
void  usb_resume_host(void);
void custom_set_usb_cfg_params(custom_cfg_t *p);
void user_resume_host();
void usb_host_cmd_proc(u8 *pkt);
void generateRandomNum(int len, unsigned char *data);

/* Disable C linkage for C++ Compilers: */
#if defined(__cplusplus)
}
#endif
