/********************************************************************************************************
 * @file     aaa_24g_app.c
 *
 * @brief    for TLSR chips
 *
 * @author	 telink
 * @date     Sep. 30, 2010
 *
 * @par      Copyright (c) 2010, Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/
#include "AAA_public_config.h"

extern void mouse_task_when_rf();

typedef enum
{
    STATE_POWERON = 0,
    STATE_SYNCING,
    STATE_PAIRING,
    STATE_NORMAL,
    //STATE_SUSPEND,
    //STATE_DEEPSLEEP,
    //STATE_EMI,

} MOUSE_MODE;

u32 device_seq_no = 0;

ENC_RF_DATA ms_rf_dat;


#define 	MOUSE_BUTTON_DEBOUNCE			3

#define		RF_SYNC_PKT_TH_NUM				0xff

#define 	RF_SYNC_PKT_TX_NUM  			((RF_SYNC_PKT_TH_NUM << 1) + RF_SYNC_PKT_TH_NUM )


#define SERACH_DONGLE_TIME   1250  //UINT 8mS


#define DEEP_RETION_TIME 4//UINT 1S

#define BASE_SUSPEND_TIME    8  //UINT 1MS
#define LONG_SUSPEND_TIME    100 //UINT 1MS


#define PARING_POWER_ON_CNT     64

#define EMPTY_PACKET_AAA     300

#if(!MOUSE_PIPE1_DATA_WITH_DID)
    _attribute_data_retention_user u8 pipe1_send_id_flg ;
#endif

_attribute_data_retention_user u8 private_key[16] =
{
    0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10
};

_attribute_data_retention_user u8 data_num = 0;
_attribute_data_retention_user mouse_data_t last_ms_data;

//_attribute_data_retention_user u8 device_paired_flag = 0;
_attribute_data_retention_user u8 device_status = 0;
_attribute_data_retention_user u32 dongle_id;

_attribute_data_retention_user u8 mouse_send_need_f = 1;

_attribute_data_retention_user u16 no_ack = 0;

_attribute_data_retention_user u16 power_on_cnt = 0;

_attribute_data_retention_user rf_packet_pairing_t	pkt_pairing =
{
    sizeof(rf_packet_pairing_t) - 4,	// dma_len

    sizeof(rf_packet_pairing_t) - 5,	// rf_len
    RF_PROTO_BYTE,						// proto

    PKT_FLOW_DIR,	                    // flow
    FRAME_TYPE_MOUSE,					// type
    //		PIPE0_CODE,			// gid0
    0,					// rssi
    0,					// per
    0,					// seq_no
    0,					// reserved
    0x0c0c0c0c			// device id
};
_attribute_data_retention_user rf_packet_mouse_t	pkt_mouse =
{
    sizeof(rf_packet_mouse_t) - 4,	// dma_len
    sizeof(rf_packet_mouse_t) - 5,	// rf_len,	28-5 = 0x17
    RF_PROTO_BYTE,					// proto,	0x51
    PKT_FLOW_DIR,					// flow,	0x80
    FRAME_TYPE_MOUSE,				// type,	0x01
    //		U32_MAX,			// gid0
    0,					// rssi
    0,					// per
    0,					// seq_no
    1,					// number of frame
};

_attribute_data_retention_user u32 tick_loop = 0;
_attribute_data_retention_user u32 wakeup_next_tick = 0;
_attribute_data_retention_user u16 wake_timer = BASE_SUSPEND_TIME;
_attribute_data_retention_user u8 count = 0;
_attribute_data_retention_user u8 pair_success_flag = 0;

_attribute_ram_code_ u8  rf_rx_process(rf_packet_ack_pairing_t *p_pkt)
{
    if (p_pkt->proto == RF_PROTO_BYTE)
    {
#if ((AES_METHOD==0)||(AES_METHOD==1))
        if ((p_pkt->type == FRAME_TYPE_ACK) && (p_pkt->did == pkt_pairing.did))
#elif ((AES_METHOD==2))
        if ((p_pkt->type == FRAME_TYPE_ACK) && (memcmp(&p_pkt->did, &pkt_pairing.did, 16)==0))
#endif
        {
            dongle_id = p_pkt->gid1;
            pair_success_flag = 1;
            return 1;
        }
        ///////////// PIPE1: ACK /////////////////////////////
        else if (p_pkt->type == FRAME_TYPE_ACK_MOUSE)
        {
#if(!MOUSE_PIPE1_DATA_WITH_DID)
            pipe1_send_id_flg = 0;
#endif
            return 1;
        }
        else if (p_pkt->type == FRAME_TYPE_ACK_KEYBOARD)
        {
#if(!MOUSE_PIPE1_DATA_WITH_DID)
            pipe1_send_id_flg = 0;
#endif
            return 1;
        }
#if(!MOUSE_PIPE1_DATA_WITH_DID)
        else if (p_pkt->type == FRAME_AUTO_ACK_MOUSE_ASK_ID) //fix auto bug
        {
            pipe1_send_id_flg = 1;
            return 1;
        }
#endif
#if(!MOUSE_PIPE1_DATA_WITH_DID)
        else if (p_pkt->type == FRAME_AUTO_ACK_KB_ASK_ID)
        {
            pipe1_send_id_flg = 1;//fix auto bug
            return 1;
        }
#endif
        ////////// end of PIPE1 /////////////////////////////////////
    }
    return 0;
}

#if TEST_LOST_RATE
    u16 tx_pair_cnt = 0;
    u16 tx_normal_cnt = 0;
    extern  volatile unsigned int  rf_rx_cnt;
    extern  volatile unsigned int  rf_rx_timeout_cnt;
    extern  volatile unsigned int  user_rx_timeout_cnt;
    extern  volatile unsigned int  user_rx_right_data_cnt;
#endif
void adv_syn_proc()
{
#if (TEST_LOST_RATE==0)
    if (user_cfg.rf_vid != U16_MAX)
    {
        set_pair_access_code(rf_access_code_16to32(user_cfg.rf_vid));
    }
    else
    {
        //pipe0_id = 0xe7ba;
        set_pair_access_code(0x39517695);
    }
#else
    set_pair_access_code(rf_access_code_16to32(0x3333));
#endif
    if (device_status == STATE_PAIRING)
    {
        rf_set_power_level_index(user_cfg.paring_tx_power);
        pkt_pairing.flow = PKT_FLOW_PARING;
    }
#if 0
    else if (device_status == STATE_SYNCING)
    {
        rf_set_power_level_index(user_cfg.tx_power);

        if (power_on_cnt < PARING_POWER_ON_CNT)
        {
            power_on_cnt++;
            pkt_pairing.flow = PKT_FLOW_TOKEN | PKT_FLOW_PARING;
        }
        else
        {
            pkt_pairing.flow = PKT_FLOW_TOKEN;
        }
    }
#endif
    wake_timer = BASE_SUSPEND_TIME;
   // pkt_pairing.type = FRAME_TYPE_MOUSE;
    pair_success_flag = 0;
    has_new_key_event = 0;
    clear_fifo();
    if (device_send_packet((u8 *)&pkt_pairing, 3, 700) && pair_success_flag)
    {
#if 1
        no_ack = 0;
        device_status = STATE_SYNCING;
        //device_paired_flag = 1;
#if TEST_LOST_RATE
        tx_normal_cnt = 0;
        rf_rx_cnt = 0;
        rf_rx_timeout_cnt = 0;
        user_rx_right_data_cnt = 0;
        user_rx_timeout_cnt = 0;
#endif
	#if ((AES_METHOD==0)||(AES_METHOD==1))
        if (flash_dev_info.dongle_id != dongle_id)
	#endif
        {
            flash_dev_info.dongle_id = dongle_id;
			#if (AES_METHOD==2)
            memcpy((u8*)&flash_dev_info.key[0], (u8*)&private_key[4], 12);
			#endif

            save_dev_info_flash();
        }
        reset_idle_status();
        clear_pair_flag();

#endif
    }
    else
    {
        no_ack++;

#if D24G_ADV_ENTER_DEEPSLEEP_AFTER_TIME_OUT_ENABLE_AAA
        if ((idle_count >= D24G_ADV_TIMER_OUT))
        {
            enter_deep_aaa();
        }
#endif
    }

}
void d24_start_pair()
{
    set_pair_flag();
    user_reboot(CLEAR_FLAG_ANA_AAA);
}




void aes_user_encryption(u8 *key, u8 *plaintext, u8 *encrypted_data)
{
    //ecb
    u8 rplaintext[16], rkey[16];
    swapX(plaintext, rplaintext, 16);
    swapX(key, rkey, 16);
    aes_ll_encryption(rkey, rplaintext, encrypted_data);
}

void aes_user_decryption(u8 *key, u8 *encrypted_data, u8 *decrypted_data)
{
    //ecb
    u8 rencrypted_data[16], rkey[16];
    swapX(key, rkey, 16);
    swapX(encrypted_data, rencrypted_data, 16);
    aes_ll_decryption(rkey, rencrypted_data, decrypted_data);
}

    //aes Key
#if 0

void d24g_show_mode()
{
    if ((deep_flag == POWER_ON_ANA_AAA) || (deep_flag == MODE_CHANGE_REBOOT_ANA_AAA))
    {
        if (deep_flag == MODE_CHANGE_REBOOT_ANA_AAA)
        {
            write_deep_ana0(CLEAR_FLAG_ANA_AAA);
        }
#if BLT_APP_LED_ENABLE
        led_24g_mode_display();
#endif
    }
}
#endif
void d24_user_init()
{
    //d24g_show_mode();
    /*****************   set up did(device id)    ********************/
    memcpy(&pkt_pairing.rssi, &output_dev_info.bin_crc, 4);

   
    ms_rf_dat.device_id = (user_cfg.dev_mac&(0x00ffffff))|(FRAME_TYPE_MOUSE<<24);
    // ms_rf_dat.device_id =0x01223344;
#if(MOUSE_PIPE1_DATA_WITH_DID)
	pkt_mouse.did = ms_rf_dat.device_id;
#endif
    dongle_id = flash_dev_info.dongle_id ;
	memcpy((u8*)&private_key[0],(u8*)&ms_rf_dat.device_id ,4);


#if  ENTER_PAIR_WHEN_NEVER_PAIRED_ENABLE
    if ((dongle_id== U32_MAX) || (dongle_id== 0))
    {
        set_pair_flag();
    }
	else
	{
		has_been_paired_flag=1;
	}
#endif

    if (pair_flag)
    {
        device_status = STATE_PAIRING;
	#if((AES_METHOD==1)||(AES_METHOD==0))
		pkt_pairing.did=ms_rf_dat.device_id;
	#elif (AES_METHOD==2)
		generateRandomNum(12, &private_key[4]);
		aes_user_encryption(pub_key,private_key,(u8*)&pkt_pairing.did);
	#endif
    }
    else
    {
		memcpy((u8*)&private_key[4], (u8*)&flash_dev_info.key[0], 12);
		if(deep_flag!=DEEP_SLEEP_ANA_AAA)
		{
			device_status = STATE_SYNCING;
    	}
    	else
    	{
        	device_status = STATE_NORMAL;
		}
    }
    mouse_send_need_f=1;
#if (BATT_CHECK_ENABLE)

#endif
    wakeup_next_tick = clock_time();
    reset_idle_status();
}

#if 0
void ui_loop_24g11()
{
    u8 rf_send_success = 1;

#if DO_TASK_WHEN_RF_IN_24G_MODE_ENALBE
    p_task_when_rf = mouse_task_when_rf;
#else
    mouse_task_when_rf();
#endif

    get_24g_data_report_aaa();
    loop_cnt++;
    if (has_new_key_event)
    {
        reset_idle_status();
        has_new_key_event = 0;
    }
    idle_status_poll();

    if (device_status == STATE_NORMAL)
    {
#if 0
        ms_data.x = 2;
        mouse_send_need_f = 1;
#else
        if (ms_data.btn || ms_data.x || ms_data.y || ms_data.wheel || (loop_cnt < EMPTY_PACKET_AAA))
        {
#if DOUBLE_CLICK_LEFT_FUN_ENABLE
            if (has_double_click_left)
            {
                double_click_left_24G_mode();
            }
            else
#endif
            {
                ms_buf.btn = ms_data.btn;
            }
            ms_buf.x = ms_data.x;
            ms_buf.y = ms_data.y;
            ms_buf.wheel = ms_data.wheel;
            mouse_send_need_f = 1;
            wake_timer = BASE_SUSPEND_TIME;
        }

#endif
        if (mouse_send_need_f)
        {
            u8 src[16] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
            if (device_seq_no != 0)
            {
                device_seq_no++;
            }
            pkt_mouse.seq_no++;
            //generateRandomNum(4,&src[12]);
            memcpy(&src[12], &device_seq_no, 4);
            if (data_num == 0)
            {
                memcpy(src, &ms_buf.btn, sizeof(mouse_data_t));
                data_num = 1;
            }
            else
            {
                last_ms_data.wheel = 0;
                memcpy(src, &last_ms_data.btn, sizeof(mouse_data_t));
                memcpy(&src[sizeof(mouse_data_t)], &ms_buf.btn, sizeof(mouse_data_t));
                data_num = 2;
            }
            memcpy(&last_ms_data.btn, &ms_buf.btn, sizeof(mouse_data_t));
#if AES_USER_ENABLE
            if (user_cfg.aes_enable)
            {
                u8 encrypted_data[16] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
                aes_user_encryption(encKey, src, encrypted_data);
                memcpy(pkt_mouse.data, encrypted_data, 16);
                pkt_mouse.pno = 0x80 | data_num;
            }
            else
#endif
            {
                memcpy(pkt_mouse.data, src, sizeof(mouse_data_t)*data_num);
                pkt_mouse.pno = data_num;
            }
#if(!MOUSE_PIPE1_DATA_WITH_DID)
            fix_auto_pair_dongle();
#endif
            set_data_access_code(flash_dev_info.dongle_id);
            rf_set_power_level_index(user_cfg.tx_power);

            if (device_send_packet((u8 *)&pkt_mouse, 3, 600))
            {
                mouse_send_need_f = 0;
                ms_data.wheel = 0;
                data_num = 0;
                device_seq_no++;
            }
            else
            {
                rf_send_success = 0;
            }

        }
        if (rf_send_success)
        {
            no_ack = 0;
        }
        else
        {
            no_ack++;
            if (no_ack > SERACH_DONGLE_TIME)
            {
                enter_deep_aaa();
            }
        }

        if (mouse_send_need_f || btn_value || DEVICE_LED_BUSY || (no_ack > 0))
        {
            wake_timer = BASE_SUSPEND_TIME;
        }
#if D24G_CONNECT_ENTER_DEEPSLEEP_AFTER_TIME_OUT_ENABLE_AAA
        if ((idle_count >= D24G_CONNECT_TIME_OUT))
        {
            enter_deep_aaa();
        }
#endif
#if TEST_LOST_RATE
        tx_normal_cnt++;
        if (tx_normal_cnt >= 1000)
        {
            gpio_set_output_en(PIN_24G_LED, 1);
            gpio_write(PIN_24G_LED, LED_ON_AAA);
            while (1);
        }
#endif
    }
    else if (device_status <= STATE_PAIRING)
    {
        adv_syn_proc();
#if TEST_LOST_RATE
        tx_pair_cnt++;
        if (tx_pair_cnt >= 1000)
        {
            gpio_set_output_en(PIN_24G_LED, 1);
            gpio_write(PIN_24G_LED, LED_ON_AAA);
            while (1);
        }
#endif
    }
#if DO_TASK_WHEN_RF_IN_24G_MODE_ENALBE
    if (p_task_when_rf != NULL)
    {
        (*p_task_when_rf)();
    }
#endif
    device_led_process();
}

#endif

void ui_loop_24g()
{

#if DO_TASK_WHEN_RF_IN_24G_MODE_ENALBE
    p_task_when_rf = mouse_task_when_rf;
#else
    mouse_task_when_rf();
#endif

    get_24g_data_report_aaa();
    loop_cnt++;
    if (has_new_key_event)
    {
        reset_idle_status();
        has_new_key_event = 0;
    }
    idle_status_poll();

	if ((device_status == STATE_NORMAL)||(device_status == STATE_SYNCING))
    {
		if(device_status == STATE_SYNCING)
		{
			ms_data.wheel=0;// for8258  power on 
		}
        mouse_send_need_f |= check_fifo_has_data();
        if (ms_data.btn || ms_data.x || ms_data.y || ms_data.wheel || (loop_cnt < EMPTY_PACKET_AAA))
        {

            mouse_send_need_f = 1;
            wake_timer = BASE_SUSPEND_TIME;
            push_data_fifo_aaa(&ms_data.btn, HAS_MOUSE_REPORT, sizeof(mouse_data_t));
        }

        if (mouse_send_need_f)
        {
            data_num = 0;
            u8 rptr = tx_fifo_aaa.rptr;
#if ((AES_METHOD==0)||(AES_METHOD==1))
            for (int i = 0; rptr != tx_fifo_aaa.wptr && i < 2; i++)
#elif (AES_METHOD==2)
            for (int i = 0; rptr != tx_fifo_aaa.wptr && i < 3; i++)
#endif
            {
                memcpy(&ms_rf_dat.ms_buf[sizeof(mouse_data_t) * data_num++], &tx_fifo_aaa.fifo[rptr & (TX_FIFO_NUM_AAA - 1)][2], sizeof(mouse_data_t));
                rptr++;
            }


#if (AES_METHOD==0)
            memcpy((u8 *)&pkt_mouse.dat.device_id, &ms_rf_dat.ms_buf[0], 12);
            pkt_mouse.pno = data_num;
			pkt_mouse.dma_len=sizeof(rf_packet_mouse_t) - 20;
			pkt_mouse.rf_len=sizeof(rf_packet_mouse_t) - 21;
#elif (AES_METHOD==1)
            memcpy((u8 *)&pkt_mouse.dat.device_id, &ms_rf_dat.ms_buf[0], 12);
            generateRandomNum(4, &ms_rf_dat.ms_buf[12]);
            aes_user_encryption(pub_key, &ms_rf_dat.ms_buf[0], (u8 *)&pkt_mouse.dat.device_id);
            pkt_mouse.pno = 0x80 | data_num;
			pkt_mouse.dma_len=sizeof(rf_packet_mouse_t) - 20;
			pkt_mouse.rf_len=sizeof(rf_packet_mouse_t) - 21;
#elif (AES_METHOD==2)
            generateRandomNum(4, &ms_rf_dat.rand_dat[0]);
            ms_rf_dat.seq_no++;

            u8 *p_src = (u8 *)&ms_rf_dat.device_id;
            u8 *p_enc = (u8 *)&pkt_mouse.dat.device_id;
			aes_user_encryption(private_key, &p_src[0], &p_enc[0]);
			if(data_num>1)
			{
				aes_user_encryption(private_key, &p_src[16], &p_enc[16]);
				pkt_mouse.dma_len=sizeof(rf_packet_mouse_t) - 4;
				pkt_mouse.rf_len=sizeof(rf_packet_mouse_t) - 5;
			}
			else
			{
				pkt_mouse.dma_len=sizeof(rf_packet_mouse_t) - 20;
				pkt_mouse.rf_len=sizeof(rf_packet_mouse_t) - 21;
			}
            pkt_mouse.pno = data_num | 0x80;
#endif
            set_data_access_code(flash_dev_info.dongle_id);
            rf_set_power_level_index(user_cfg.tx_power);

            if (device_send_packet((u8 *)&pkt_mouse, 3, 600))
            {
                mouse_send_need_f = 0;
                tx_fifo_aaa.rptr = rptr;
                no_ack = 0;
				#if BLT_APP_LED_ENABLE
				if(device_status==STATE_SYNCING)
				{
					led_24g_ConnectedStatus();
				}
				#endif
				device_status=STATE_NORMAL;
                pkt_mouse.seq_no++;
            }
            else
            {
                no_ack++;
				if (no_ack>125)
				{
					device_status=STATE_SYNCING;
				}
                if (no_ack > SERACH_DONGLE_TIME)
                {
                    enter_deep_aaa();
				
				}
            }

        }

        if (mouse_send_need_f || btn_value || DEVICE_LED_BUSY || (no_ack > 0))
        {
            wake_timer = BASE_SUSPEND_TIME;
        }
#if D24G_CONNECT_ENTER_DEEPSLEEP_AFTER_TIME_OUT_ENABLE_AAA
        if ((idle_count >= D24G_CONNECT_TIME_OUT))
        {
            enter_deep_aaa();
        }
#endif
#if TEST_LOST_RATE
        tx_normal_cnt++;
        if (tx_normal_cnt >= 1000)
        {
            gpio_set_output_en(PIN_24G_LED, 1);
            gpio_write(PIN_24G_LED, LED_ON_AAA);
            while (1);
        }
#endif
    }
    else if (device_status <= STATE_PAIRING)
    {
        adv_syn_proc();
#if TEST_LOST_RATE
        tx_pair_cnt++;
        if (tx_pair_cnt >= 1000)
        {
            gpio_set_output_en(PIN_24G_LED, 1);
            gpio_write(PIN_24G_LED, LED_ON_AAA);
            while (1);
        }
#endif
    }
#if DO_TASK_WHEN_RF_IN_24G_MODE_ENALBE
    if (p_task_when_rf != NULL)
    {
        (*p_task_when_rf)();
    }
#endif
    device_led_process();
}


void d24_main_loop()
{
#if 0
    static  u32 loop_tick;
    while (1)
    {
        if (clock_time_exceed(loop_tick, 8000))
        {
            break;
        }
    }


    loop_tick = clock_time();
    ui_loop_24g();
    device_channel = get_next_channel_with_mask(ll_chn_mask, device_channel);

#else
    u32 wake_src = 0;
    connect_ok = (device_status == STATE_NORMAL) ? 1 : 0;
    if((connect_ok==0))
    {
		#if BLT_APP_LED_ENABLE
    		led_2p4_Adv_poll();
		#endif
    }

    //if (clock_time_exceed(start_tick, 4500))
    {
        //start_tick = clock_time();
        ui_loop_24g();
    }
    if (wake_timer == BASE_SUSPEND_TIME)
    {
        if (suspend_wake_up_enable)
        {
#if WHEEL_FUN_ENABLE_AAA
            wheel_set_wakeup_level_suspend(0);
#endif
#if BUTTON_FUN_ENABLE_AAA
            btn_set_wakeup_level_suspend(0);
#endif
#if SENSOR_FUN_ENABLE_AAA
            sensor_set_wakeup_level_suspend(0);
#endif
        }
        suspend_wake_up_enable = 0;
        wake_src = PM_WAKEUP_TIMER;

    }
    else
    {
        //if (suspend_wake_up_enable == 0)
        {
#if WHEEL_FUN_ENABLE_AAA
            wheel_set_wakeup_level_suspend(1);
#endif
#if BUTTON_FUN_ENABLE_AAA
            btn_set_wakeup_level_suspend(1);
#endif
#if SENSOR_FUN_ENABLE_AAA
            sensor_set_wakeup_level_suspend(1);
#endif
        }
        suspend_wake_up_enable = 1;
        device_sync = 0;
        wake_src = PM_WAKEUP_TIMER | PM_WAKEUP_PAD;
    }

    int wake_up_status = cpu_sleep_wakeup(SUSPEND_MODE, wake_src, (wakeup_next_tick + wake_timer * CLOCK_SYS_CLOCK_1MS));
    if (wake_up_status & WAKEUP_STATUS_CORE)
    {
        //reset_idle_status();
        loop_cnt = 0;
    }
    else if (wake_up_status & WAKEUP_STATUS_TIMER)
    {
        if (wake_timer == BASE_SUSPEND_TIME)
        {
            device_channel = get_next_channel_with_mask(ll_chn_mask, device_channel);
        }

    }

    wakeup_next_tick = clock_time();
    rf_drv_private_2m_init();
    wake_timer = LONG_SUSPEND_TIME;
#endif
}
