/********************************************************************************************************
 * @file     aaa_main.c
 *
 * @brief    for TLSR chips
 *
 * @author	 telink
 * @date     Sep. 30, 2010
 *
 * @par      Copyright (c) 2010, Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/

#include "AAA_public_config.h"

_attribute_data_retention_user int dev_info_idx;
_attribute_data_retention_user u8 pub_key[16] =
{
    0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F, 0x10
};


_attribute_data_retention_user OUTPUT_DEV_INFO_AAA output_dev_info =
{
    0,//u32 bin_crc;
    {2, 4, 4}, //u8
    0,//u8 sensor_type;
    0,//u8 sensor_pd1;
    0,//u8 sensor_pd2;
    0,//u8 sensor_pd3;
    MOUSE_DATA_LEN_AAA,//u8 ms_data_len;
};

_attribute_ram_code_ void irq_handler(void)
{
    if (fun_mode == RF_2M_2P4G_MODE)
    {
        irq_handle_private_2m();
    }
    else
    {
        irq_blt_sdk_handler();
    }
}

//len must 4 bytes bei beisu  beacuse *(volatile u32 *)(s_addr + idx) must 4 bytes bei beisu
int flash_info_load_aaa(u32 s_addr, u8 *d_addr,  u8 len)
{
    int idx;
    u32 buf;
    for (idx = 0; idx < 4096; idx += len)
    {
        flash_read_page((u32)(s_addr + idx), 4, (u8 *)(&buf));
        if (buf == U32_MAX)
        {
            break;
        }
    }
    idx -= len;
    if (idx < 0) 		// no binding
    {
        return idx;
    }
    flash_read_page((u32)(s_addr + idx), len, d_addr);

    if (idx > 4000) 			//3k, erase flash
    {
        flash_erase_sector((u32)s_addr);

        sleep_us(10);

        flash_write_page((u32)s_addr, len, d_addr);
        idx = 0;
    }
    return idx;
}

#if IS_SINGLE_GPIO_CHANGE_MODE

void three_switch_mode_change()
{

    //gpio_setup_up_down_resistor(PIN_MODE_SWITCH,PM_PIN_PULLUP_1M);
    //sleep_us(20);
    if (IS_BLE_MODE_AAA)
    {
        flash_dev_info.mode = RF_1M_BLE_MODE;
        //gpio_set_input_en(GPIO_MODE_SWITCH,0);

    }
    else
    {
        flash_dev_info.mode = RF_2M_2P4G_MODE;
    }
    //gpio_setup_up_down_resistor(PIN_MODE_SWITCH,PM_PIN_UP_DOWN_FLOAT);
#if 0 //test switch is good
    gpio_set_output_en(PIN_BLE_LED, 1);
    while (1)
    {
        if (IS_BLE_MODE_AAA)
        {

            gpio_write(PIN_BLE_LED, LED_ON_AAA);
        }
        else
        {
            gpio_write(PIN_24G_LED, LED_ON_AAA);
        }
    }

#endif
}
#endif
void user_config_ykq()
{
    random_generator_init();  //this is must
    flash_read_page(CFG_ADR_MAC, sizeof(custom_cfg_t), (u8 *)&user_cfg.dev_mac);
    if (user_cfg.dev_mac == U32_MAX)
    {
        generateRandomNum(4, (u8 *) &user_cfg.dev_mac);
        flash_write_page(CFG_ADR_MAC, 4, (u8 *)&user_cfg.dev_mac);
    }


    if (user_cfg.paring_tx_power == U8_MAX)
    {
        user_cfg.paring_tx_power = DEFAULT_PAIR_TX_POWER;
    }
    if (user_cfg.tx_power == U8_MAX)
    {
        user_cfg.tx_power = DEFAULT_NORMAL_TX_POWER;
    }
    if (user_cfg.emi_tx_power == U8_MAX)
    {
        user_cfg.emi_tx_power = DEFAULT_EMI_TX_POWER;
    }


    if ((user_cfg.sensor_direct == U8_MAX))
    {
		
        user_cfg.sensor_direct = SENSOR_DIRECTION_CLOCK_3;
    }
    else if (user_cfg.sensor_direct == 0)
    {
        user_cfg.sensor_direct = SENSOR_DIRECTION_CLOCK_3;
    }
    else if (user_cfg.sensor_direct == 1)
    {
        user_cfg.sensor_direct = SENSOR_DIRECTION_CLOCK_6;
    }
    else if (user_cfg.sensor_direct == 2)
    {
        user_cfg.sensor_direct = SENSOR_DIRECTION_CLOCK_9;
    }
    else if (user_cfg.sensor_direct == 3)
    {
        user_cfg.sensor_direct = SENSOR_DIRECTION_CLOCK_12;
    }
     //if(user_cfg.internal_cap!=0)
     {
	 	//blc_app_setExternalCrystalCapEnable(1);
     }
    //aes Key
    for (u8 i = 0; i < 16; i++)
    {
        if (user_cfg.pub_key[i] != 0xff)
        {
            memcpy(pub_key, &user_cfg.pub_key[0], 16);
            break;
        }
    }

    for (u8 i = 0; i < 18; i++)
    {
        if (user_cfg.device_name[i] != 0xff)
        {
            device_name_len++;
        }
        else
        {
            break;
        }
    }

}

int deepRetWakeUp;

_attribute_ram_code_ int main(void)     //must run in ramcode
{
    blc_pm_select_internal_32k_crystal();

    cpu_wakeup_init();

    deepRetWakeUp = pm_is_MCU_deepRetentionWakeup();  //MCU deep retention wakeUp

    gpio_init(!deepRetWakeUp);    //analog resistance will keep available in deepSleep mode, so no need initialize again

#if (CLOCK_SYS_CLOCK_HZ == 16000000)
    clock_init(SYS_CLK_16M_Crystal);
#elif (CLOCK_SYS_CLOCK_HZ == 24000000)
    clock_init(SYS_CLK_24M_Crystal);
#endif



    u32 bin_size = 0;
    flash_read_page(0x18, 4, (u8 *)&bin_size);
    flash_read_page((bin_size - 4), 4, (u8 *)&output_dev_info.bin_crc);

    if (!deepRetWakeUp)
    {
        deep_flag = analog_read(DEEP_ANA_REG0);
        ana_reg1_aaa = analog_read(USED_DEEP_ANA_REG1);
        pair_flag = ana_reg1_aaa & PAIR_ANA_FLG;

        user_config_ykq();
        dev_info_idx = flash_info_load_aaa(CFG_DEVICE_MODE_ADDR, (u8 *)&flash_dev_info.dongle_id, SAVE_MAX_IN_FLASH);
        if (dev_info_idx < 0)
        {
            flash_dev_info.mode = RF_2M_2P4G_MODE;
            flash_dev_info.mast_id = 0;
            flash_dev_info.slave_mac_addr[0] = 0;
            flash_dev_info.slave_mac_addr[1] = 0;
            flash_dev_info.slave_mac_addr[2] = 0;
            flash_dev_info.slave_mac_addr[3] = 0;
            flash_dev_info.dongle_id = 0;
        }
		 blc_app_loadCustomizedParameters();  //load customized freq_offset cap value

    
        switch_type_init();

        //flash_dev_info.mode=RF_1M_BLE_MODE;//for test mode
#if (BLE_SNIFF_DEBUG)
        flash_dev_info.mode = RF_1M_BLE_MODE;
#endif
        fun_mode = flash_dev_info.mode;
#if DPI_SAVE_FLASH

        flash_dpi_info.idx = flash_info_load_aaa((MOUSE_DPI_ADDR), (u8 *)&flash_dpi_info.bt_dpi, 4);

        if (flash_dpi_info.idx < 0)
#endif
        {
            flash_dpi_info.bt_dpi = 1;//default
#if DPI_SAVE_FLASH
            flash_dpi_info.d24g_dpi = 1;//default
#endif
        }
    }
    
    hw_init();
	if (deep_flag == POWER_ON_ANA_AAA)
    {
        power_on_tick = clock_time() | 1;
#if EMI_TEST_FUN_ENABLE_AAA
        emi_process();
#endif
    }

#if (BATT_CHECK_ENABLE)  //battery check must do before OTA relative operation
    user_batt_check_init();
#endif
 

    if (fun_mode == RF_1M_BLE_MODE)
    {
        rf_drv_init(RF_MODE_BLE_1M);
        if (deepRetWakeUp)
        {
            user_init_deepRetn();
        }
        else
        {
            user_init_normal();
        }
    }
    else
    {
        rf_drv_private_2m_init();//ykq change
        if (!deepRetWakeUp)
        {
            d24_user_init();
        }
    }
    blc_pm_setDeepsleepRetentionType(DEEPSLEEP_MODE_RET_SRAM_LOW32K);

    irq_enable();

    while (1)
    {
#if (MODULE_WATCHDOG_ENABLE)
        wd_clear(); //clear watch dog
#endif
#if DEBUG_TOGGLE_GPIO_ENABLE
        debug_loop_toggle();
#endif
        if (fun_mode == RF_1M_BLE_MODE)
        {
            main_loop();
        }
        else
        {
            d24_main_loop();
        }
#if (BATT_CHECK_ENABLE)
        user_batt_check_proc();
#endif


        //deepRetWakeUp = 0;
        last_btn_value = btn_value;
#if UART_PRINT_DEBUG_ENABLE
        _attribute_data_retention_user  static u32 tick = 0;
        if (clock_time_exceed(tick, 3000000))
        {
            tick = clock_time();
            // my_printf_aaa("poll mode=%x,ble_status=%x,fifoNum=%x,interval=%x,latency=%x,timeout=%x\r\n", fun_mode, ble_status_aaa, blc_ll_getTxFifoNumber(), bls_ll_getConnectionInterval(), bls_ll_getConnectionLatency(), bls_ll_getConnectionTimeout());
        }
        my_printf_aaa("fifoNum=%x\r\n",  blc_ll_getTxFifoNumber());
#endif
    }
}

